@@ -3,6 +3,7 @@
 Create ssh executor system
 '''
 # Import python libs
+from __future__ import print_function
 import os
 import tarfile
 import tempfile
@@ -21,19 +22,28 @@ import salt.client.ssh.shell
 import salt.client.ssh.wrapper
 import salt.utils
 import salt.utils.thin
+import salt.utils.verify
+import salt.utils.event
 import salt.roster
 import salt.state
 import salt.loader
 import salt.minion
 import salt.exceptions
+import salt.config
+
+try:
+    import zmq
+    HAS_ZMQ = True
+except ImportError:
+    HAS_ZMQ = False
 
 # This is just a delimiter to distinguish the beginning of salt STDOUT.  There
 # is no special meaning
 RSTR = '_edbc7885e4f9aac9b83b35999b68d015148caf467b78fa39c05f669c0ff89878'
 
 
-# This shim facilitaites remote salt-call operations
-# - Explicitly invokes bourne shell for univeral compatibility
+# This shim facilitates remote salt-call operations
+# - Uses /bin/sh for maximum compatibility
 #
 # 1. Identify a suitable python
 # 2. Test for remote salt-call and version if present
@@ -44,76 +54,129 @@ RSTR = '_edbc7885e4f9aac9b83b35999b68d015148caf467b78fa39c05f669c0ff89878'
 # Note there are two levels of formatting.
 # - First format pass inserts salt version and delimiter
 # - Second pass at run-time and inserts optional "sudo" and command
-SSH_SHIM = '''/bin/sh << 'EOF'
-      for py_candidate in \\
-            python27      \\
-            python2.7     \\
-            python26      \\
-            python2.6     \\
-            python2       \\
+SSH_SHIM = r'''/bin/sh << 'EOF'
+      #!/bin/sh
+
+      MISS_PKG=""
+
+      command -v tar >/dev/null
+      if [ $? -ne 0 ]
+      then
+         MISS_PKG="$MISS_PKG tar"
+      fi
+
+      for py_candidate in \
+            python27      \
+            python2.7     \
+            python26      \
+            python2.6     \
+            python2       \
             python        ;
       do
-         if [ $(which $py_candidate 2>/dev/null) ]
+         command -v $py_candidate >/dev/null
+         if [ $? -eq 0 ]
          then
-               PYTHON=$(which $py_candidate)
+               PYTHON="$py_candidate"
                break
          fi
       done
-      SALT=/tmp/.salt/salt-call
-      if [ {{2}} = 'md5' ]
+
+      if [ -z "$PYTHON" ]
       then
-         for md5_candidate in \\
-            md5sum            \\
-            md5               ;
+         MISS_PKG="$MISS_PKG python"
+      fi
+
+      SALT="/tmp/.salt/salt-call"
+      if [ "{{2}}" = "md5" ]
+      then
+         for md5_candidate in \
+            md5sum            \
+            md5               \
+            csum              ;
          do
-            if [ $(which $md5_candidate 2>/dev/null) ]
+            command -v $md5_candidate >/dev/null
+            if [ $? -eq 0 ]
             then
-                SUMCHECK=$(which $md5_candidate)
+                SUMCHECK="$md5_candidate"
                 break
             fi
          done
       else
-         SUMCHECK={{2}}
-      fi
-
-      if [ $SUMCHECK = '/sbin/md5' ]
-      then
-         CUT_MARK=4
-      else
-         CUT_MARK=1
-      fi
-
-      if [ -f $SALT ]
-      then
-         if [ $(cat /tmp/.salt/version) != {0} ]
+         if [ $(command -v "{{2}}" >/dev/null) ]
          then
-            {{0}} rm -rf /tmp/.salt && install -m 0700 -d /tmp/.salt
+            SUMCHECK="{{2}}"
+         fi
+      fi
+
+      if [ -z "$SUMCHECK" ]
+      then
+         MISS_PKG="$MISS_PKG md5sum/md5/csum"
+      fi
+
+      if [ -n "$MISS_PKG" ]
+      then
+            echo "The following required Packages are missing: $MISS_PKG" >&2
+            exit 127
+      fi
+
+      # MD5 check for systems with a BSD userland (includes OSX).
+      if [ "$SUMCHECK" = "md5" ]
+      then
+         SUMCHECK="md5 -q"
+      # MD5 check for AIX systems.
+      elif [ "$SUMCHECK" = "csum" ]
+      then
+         SUMCHECK="csum -h MD5"
+      fi
+
+      if [ -f "$SALT" ]
+      then
+         if [ "$(cat /tmp/.salt/version)" != "{0}" ]
+         then
+            {{0}} rm -rf /tmp/.salt && mkdir -m 0700 -p /tmp/.salt
             if [ $? -ne 0 ]; then
                 exit 1
             fi
+            if [ "$(stat -c %a /tmp/.salt)" != "700" ]
+            then
+                echo "Permissions for /tmp/.salt not correct. Exiting!"
+                exit 1
+            fi
+            if [ "$(stat -c %u /tmp/.salt)" != "0" ] 
+            then
+                echo "Ownership for /tmp/.salt not correct. Exiting!"
+                exit 1
+            fi
             echo "{1}"
             echo "deploy"
             exit 1
          fi
       else
-         PY_TOO_OLD=$($PYTHON -c 'import sys; print sys.hexversion < 0x02060000')
-         if [ $PY_TOO_OLD = 'True' ];
+         PY_TOO_OLD="$($PYTHON -c "import sys; print sys.hexversion < 0x02060000")"
+         if [ "$PY_TOO_OLD" = "True" ];
          then
             echo "Python too old" >&2
             exit 1
          fi
          if [ -f /tmp/.salt/salt-thin.tgz ]
          then
-             [ $($SUMCHECK /tmp/.salt/salt-thin.tgz | cut -f$CUT_MARK -d' ') = {{3}} ] && {{0}} tar opxzvf /tmp/.salt/salt-thin.tgz -C /tmp/.salt
+             if [ "$($SUMCHECK /tmp/.salt/salt-thin.tgz | cut -f1 -d\ )" = "{{3}}" ]
+             then
+                 cd /tmp/.salt/ && gunzip -c salt-thin.tgz | {{0}} tar opxvf -
+             else
+                 echo "Mismatched checksum for /tmp/.salt/salt-thin.tgz" >&2
+                 exit 1
+             fi
          else
-             install -m 0700 -d /tmp/.salt
+             rm -rf /tmp/.salt && mkdir -m 0700 -p /tmp/.salt
              echo "{1}"
              echo "deploy"
              exit 1
          fi
       fi
+      echo "{{4}}" > /tmp/.salt/minion
       echo "{1}"
-      {{0}} $PYTHON $SALT --local --out json -l quiet {{1}}
+      {{0}} $PYTHON $SALT --local --out json -l quiet {{1}} -c /tmp/.salt
 EOF'''.format(salt.__version__, RSTR)
 
 log = logging.getLogger(__name__)
@@ -125,6 +188,15 @@ class SSH(object):
     '''
     def __init__(self, opts):
         self.verify_env()
+        pull_sock = os.path.join(opts['sock_dir'], 'master_event_pull.ipc')
+        if os.path.isfile(pull_sock) and HAS_ZMQ:
+            self.event = salt.utils.event.get_event(
+                    'master',
+                    opts['sock_dir'],
+                    opts['transport'],
+                    listen=False)
+        else:
+            self.event = None
         self.opts = opts
         tgt_type = self.opts['selected_target_option'] \
                 if self.opts['selected_target_option'] else 'glob'
@@ -143,13 +215,32 @@ class SSH(object):
         if not os.path.isfile(priv):
             salt.client.ssh.shell.gen_key(priv)
         self.defaults = {
-                'user': self.opts.get('ssh_user', 'root'),
-                'port': self.opts.get('ssh_port', '22'),
-                'passwd': self.opts.get('ssh_passwd', ''),
-                'priv': priv,
-                'timeout': self.opts.get('ssh_timeout', 60),
-                'sudo': self.opts.get('ssh_sudo', False),
-                }
+            'user': self.opts.get(
+                'ssh_user',
+                salt.config.DEFAULT_MASTER_OPTS['ssh_user']
+            ),
+            'port': self.opts.get(
+                'ssh_port',
+                salt.config.DEFAULT_MASTER_OPTS['ssh_port']
+            ),
+            'passwd': self.opts.get(
+                'ssh_passwd',
+                salt.config.DEFAULT_MASTER_OPTS['ssh_passwd']
+            ),
+            'priv': priv,
+            'timeout': self.opts.get(
+                'ssh_timeout',
+                salt.config.DEFAULT_MASTER_OPTS['ssh_timeout']
+            ) + self.opts.get(
+                'timeout',
+                salt.config.DEFAULT_MASTER_OPTS['timeout']
+            ),
+            'sudo': self.opts.get(
+                'ssh_sudo',
+                salt.config.DEFAULT_MASTER_OPTS['ssh_sudo']
+            ),
+        }
+        self.serial = salt.payload.Serial(opts)
 
     def verify_env(self):
         '''
@@ -161,7 +252,7 @@ class SSH(object):
 
     def get_pubkey(self):
         '''
-        Return the keystring for the SSH public key
+        Return the key string for the SSH public key
         '''
         priv = self.opts.get(
                 'ssh_priv',
@@ -173,7 +264,7 @@ class SSH(object):
                 )
         pub = '{0}.pub'.format(priv)
         with open(pub, 'r') as fp_:
-            return '{0} root@master'.format(fp_.read().split()[1])
+            return '{0} rsa root@master'.format(fp_.read().split()[1])
 
     def key_deploy(self, host, ret):
         '''
@@ -203,6 +294,7 @@ class SSH(object):
         '''
         The ssh-copy-id routine
         '''
+        ret = {}
         arg_str = 'ssh.set_auth_key {0} {1}'.format(
                 target.get('user', 'root'),
                 self.get_pubkey())
@@ -224,7 +316,7 @@ class SSH(object):
                     self.opts['arg_str'],
                     host,
                     **target)
-            stdout, stderr = single.cmd_block()
+            stdout, stderr, retcode = single.cmd_block()
             try:
                 data = salt.utils.find_json(stdout)
                 return {host: data.get('local', data)}
@@ -245,31 +337,33 @@ class SSH(object):
                 host,
                 **target)
         ret = {'id': single.id}
-        stdout, stderr = single.run()
+        stdout, stderr, retcode = single.run()
         if stdout.startswith('deploy'):
             single.deploy()
-            stdout, stderr = single.run()
+            stdout, stderr, retcode = single.run()
         # This job is done, yield
         try:
-            if not stdout and stderr:
-                if 'Permission denied' in stderr:
-                    ret['ret'] = 'Permission denied'
-                else:
-                    ret['ret'] = stderr
+            data = salt.utils.find_json(stdout)
+            if len(data) < 2 and 'local' in data:
+                ret['ret'] = data['local']
             else:
-                data = salt.utils.find_json(stdout)
-                if len(data) < 2 and 'local' in data:
-                    ret['ret'] = data['local']
-                else:
-                    ret['ret'] = data
+                ret['ret'] = {
+                    'stdout': stdout,
+                    'stderr': stderr,
+                    'retcode': retcode,
+                }
         except Exception:
-            ret['ret'] = stdout
+            ret['ret'] = {
+                'stdout': stdout,
+                'stderr': stderr,
+                'retcode': retcode,
+            }
         que.put(ret)
 
     def handle_ssh(self):
         '''
         Spin up the needed threads or processes and execute the subsequent
-        rouintes
+        routines
         '''
         que = multiprocessing.Queue()
         running = {}
@@ -285,7 +379,7 @@ class SSH(object):
                     init = True
                     continue
                 for default in self.defaults:
-                    if not default in self.targets[host]:
+                    if default not in self.targets[host]:
                         self.targets[host][default] = self.defaults[default]
                 args = (
                         que,
@@ -328,17 +422,73 @@ class SSH(object):
         for ret in self.handle_ssh():
             yield ret
 
+    def cache_job(self, jid, id_, ret):
+        '''
+        Cache the job information
+        '''
+        jid_dir = salt.utils.jid_dir(
+                jid,
+                self.opts['cachedir'],
+                self.opts['hash_type']
+                )
+        if not os.path.isdir(jid_dir):
+            log.error(
+                'An inconsistency occurred, a job was received with a job id '
+                'that is not present on the master: {0}'.format(jid)
+            )
+            return False
+        if os.path.exists(os.path.join(jid_dir, 'nocache')):
+            return
+        hn_dir = os.path.join(jid_dir, id_)
+        if not os.path.isdir(hn_dir):
+            os.makedirs(hn_dir)
+        # Otherwise the minion has already returned this jid and it should
+        # be dropped
+        else:
+            log.error(
+                'An extra return was detected from minion {0}, please verify '
+                'the minion, this could be a replay attack'.format(
+                    id_
+                )
+            )
+            return False
+
+        self.serial.dump(
+            ret,
+            # Use atomic open here to avoid the file being read before it's
+            # completely written to. Refs #1935
+            salt.utils.atomicfile.atomic_open(
+                os.path.join(hn_dir, 'return.p'), 'w+b'
+            )
+        )
+
     def run(self):
         '''
         Execute the overall routine
         '''
+        jid = salt.utils.prep_jid(
+                self.opts['cachedir'],
+                self.opts['hash_type'],
+                self.opts['user'])
+        if self.opts.get('verbose'):
+            msg = 'Executing job with jid {0}'.format(jid)
+            print(msg)
+            print('-' * len(msg) + '\n')
+            print('')
         for ret in self.handle_ssh():
             host = ret.keys()[0]
+            #self.cache_job(jid, host, ret)
             ret = self.key_deploy(host, ret)
             salt.output.display_output(
                     ret,
                     self.opts.get('output', 'nested'),
                     self.opts)
+            if self.event:
+                self.event.fire_event(
+                        ret,
+                        salt.utils.event.tagify(
+                            [jid, 'ret', host],
+                            'job'))
 
 
 class Single(object):
@@ -365,7 +515,7 @@ class Single(object):
             **kwargs):
         self.opts = opts
         self.arg_str = arg_str
-        self.fun, self.arg = self.__arg_comps()
+        self.fun, self.arg, self.kwargs = self.__arg_comps()
         self.id = id_
 
         args = {'host': host,
@@ -377,7 +527,11 @@ class Single(object):
                 'sudo': sudo,
                 'tty': tty}
         self.shell = salt.client.ssh.shell.Shell(opts, **args)
-        self.minion_config = yaml.dump({'root_dir': '/tmp/.salt/running_data'})
+        self.minion_config = yaml.dump(
+                {
+                    'root_dir': '/tmp/.salt/running_data',
+                    'id': self.id,
+                }).strip()
         self.target = kwargs
         self.target.update(args)
         self.serial = salt.payload.Serial(opts)
@@ -389,8 +543,16 @@ class Single(object):
         '''
         comps = self.arg_str.split()
         fun = comps[0] if comps else ''
-        arg = comps[1:]
-        return fun, arg
+        args = comps[1:]
+        s_args = []
+        kws = {}
+        for arg in args:
+            if '=' in arg:
+                (key, val) = arg.split('=')
+                kws[key] = val
+            else:
+                s_args.append(arg)
+        return fun, s_args, kws
 
     def deploy(self):
         '''
@@ -412,25 +574,27 @@ class Single(object):
         If a (re)deploy is needed, then retry the operation after a deploy
         attempt
 
-        Returns tuple of (stdout, stderr)
+        Returns tuple of (stdout, stderr, retcode)
         '''
-
-        stdout, stderr = None, None
+        stdout = stderr = retcode = None
+        arg_str = self.arg_str
 
         if self.opts.get('raw_shell'):
-            stdout, stderr = self.shell.exec_cmd(self.arg_str)
+            if not arg_str.startswith(('"', "'")) and not arg_str.endswith(('"', "'")):
+                arg_str = "'{0}'".format(arg_str)
+            stdout, stderr, retcode = self.shell.exec_cmd(arg_str)
 
         elif self.fun in self.wfuncs:
-            stdout, stderr = self.run_wfunc()
+            stdout, stderr, retcode = self.run_wfunc()
 
         else:
-            stdout, stderr = self.cmd_block()
+            stdout, stderr, retcode = self.cmd_block()
 
         if stdout.startswith('deploy') and not deploy_attempted:
             self.deploy()
             return self.run(deploy_attempted=True)
 
-        return stdout, stderr
+        return stdout, stderr, retcode
 
     def run_wfunc(self):
         '''
@@ -449,7 +613,7 @@ class Single(object):
             refresh = True
         else:
             passed_time = (time.time() - os.stat(datap).st_mtime) / 60
-            if (passed_time > self.opts.get('cache_life', 60)):
+            if passed_time > self.opts.get('cache_life', 60):
                 refresh = True
         if self.opts.get('refresh_cache'):
             refresh = True
@@ -460,8 +624,11 @@ class Single(object):
                 self.opts,
                 self.id,
                 **self.target)
-            default_opts = pre_wrapper['test.opts_pkg']()
-            opts_pkg = dict(default_opts.items() + self.opts.items())
+            opts_pkg = pre_wrapper['test.opts_pkg']()
+            opts_pkg['file_roots'] = self.opts['file_roots']
+            opts_pkg['pillar_roots'] = self.opts['pillar_roots']
+            # Use the ID defined in the roster file
+            opts_pkg['id'] = self.id
             pillar = salt.pillar.Pillar(
                     opts_pkg,
                     opts_pkg['grains'],
@@ -471,7 +638,7 @@ class Single(object):
             pillar_data = pillar.compile_pillar()
 
             # TODO: cache minion opts in datap in master.py
-            with salt.utils.fopen(datap, 'w+') as fp_:
+            with salt.utils.fopen(datap, 'w+b') as fp_:
                 fp_.write(
                         self.serial.dumps(
                             {'opts': opts_pkg,
@@ -479,7 +646,7 @@ class Single(object):
                                 'pillar': pillar_data}
                             )
                         )
-        with salt.utils.fopen(datap, 'r') as fp_:
+        with salt.utils.fopen(datap, 'rb') as fp_:
             data = self.serial.load(fp_)
         opts = data.get('opts', {})
         opts['grains'] = data.get('grains')
@@ -490,12 +657,12 @@ class Single(object):
             **self.target)
         self.wfuncs = salt.loader.ssh_wrapper(opts, wrapper)
         wrapper.wfuncs = self.wfuncs
-        ret = json.dumps(self.wfuncs[self.fun](*self.arg))
-        return ret, ''
+        ret = json.dumps(self.wfuncs[self.fun](*self.arg, **self.kwargs))
+        return ret, '', None
 
     def cmd(self):
         '''
-        Prepare the precheck command to send to the subsystem
+        Prepare the pre-check command to send to the subsystem
         '''
         # 1. check if python is on the target
         # 2. check is salt-call is on the target
@@ -515,22 +682,25 @@ class Single(object):
                 sudo,
                 self.arg_str,
                 self.opts['hash_type'],
-                thin_sum)
-        for stdout, stderr in self.shell.exec_nb_cmd(cmd):
-            yield stdout, stderr
+                thin_sum,
+                self.minion_config)
+        for stdout, stderr, retcode in self.shell.exec_nb_cmd(cmd):
+            yield stdout, stderr, retcode
 
     def cmd_block(self, is_retry=False):
         '''
-        Prepare the precheck command to send to the subsystem
+        Prepare the pre-check command to send to the subsystem
         '''
         # 1. check if python is on the target
         # 2. check is salt-call is on the target
         # 3. deploy salt-thin
         # 4. execute command
         if self.arg_str.startswith('cmd.run'):
-            cmd_args = ' '.join(self.arg_str.split()[1:])
+            arg_split = self.arg_str.split()
+            cmd = arg_split[0]
+            cmd_args = ' '.join(arg_split[1:])
             if not cmd_args.startswith("'") and not cmd_args.endswith("'"):
-                self.arg_str = "cmd.run '{0}'".format(cmd_args)
+                self.arg_str = "{0} '{1}'".format(cmd, cmd_args)
         sudo = 'sudo' if self.target['sudo'] else ''
         thin_sum = salt.utils.thin.thin_sum(
                 self.opts['cachedir'],
@@ -539,44 +709,51 @@ class Single(object):
                 sudo,
                 self.arg_str,
                 self.opts['hash_type'],
-                thin_sum)
-        log.debug("Performing shimmed command as follows:\n{0}".format(cmd))
-        stdout, stderr = self.shell.exec_cmd(cmd)
+                thin_sum,
+                self.minion_config)
+        log.debug('Performing shimmed command as follows:\n{0}'.format(cmd))
+        stdout, stderr, retcode = self.shell.exec_cmd(cmd)
 
-        log.debug("STDOUT {1}\n{0}".format(stdout, self.target['host']))
-        log.debug("STDERR {1}\n{0}".format(stderr, self.target['host']))
+        log.debug('STDOUT {1}\n{0}'.format(stdout, self.target['host']))
+        log.debug('STDERR {1}\n{0}'.format(stderr, self.target['host']))
+        log.debug('RETCODE {1}\n{0}'.format(retcode, self.target['host']))
 
-        error = self.categorize_shim_errors(stdout, stderr)
+        error = self.categorize_shim_errors(stdout, stderr, retcode)
         if error:
-            return "ERROR: {0}".format(error), stderr
+            return 'ERROR: {0}'.format(error), stderr, retcode
 
         if RSTR in stdout:
             stdout = stdout.split(RSTR)[1].strip()
         if stdout.startswith('deploy'):
             self.deploy()
-            stdout, stderr = self.shell.exec_cmd(cmd)
+            stdout, stderr, retcode = self.shell.exec_cmd(cmd)
             if RSTR in stdout:
                 stdout = stdout.split(RSTR)[1].strip()
 
-        return stdout, stderr
+        return stdout, stderr, retcode
 
-    def categorize_shim_errors(self, stdout, stderr):
-        perm_error_fmt = "Permissions problem, target user may need "\
-                         "to be root or use sudo:\n {0}"
+    def categorize_shim_errors(self, stdout, stderr, retcode):
+        # Unused stdout and retcode for now but these may be used to
+        # categorize errors
+        _ = stdout
+        _ = retcode
+
+        perm_error_fmt = 'Permissions problem, target user may need '\
+                         'to be root or use sudo:\n {0}'
         if stderr.startswith('Permission denied'):
             return None
         errors = [
-            ("sudo: no tty present and no askpass program specified",
-                "sudo expected a password, NOPASSWD required"),
-            ("Python too old",
-                "salt requires python 2.6 or better on target hosts"),
-            ("sudo: sorry, you must have a tty to run sudo",
-                "sudo is configured with requiretty"),
-            ("Failed to open log file",
+            ('sudo: no tty present and no askpass program specified',
+                'sudo expected a password, NOPASSWD required'),
+            ('Python too old',
+                'salt requires python 2.6 or better on target hosts'),
+            ('sudo: sorry, you must have a tty to run sudo',
+                'sudo is configured with requiretty'),
+            ('Failed to open log file',
                 perm_error_fmt.format(stderr)),
-            ("Permission denied:.*/salt",
+            ('Permission denied:.*/salt',
                 perm_error_fmt.format(stderr)),
-            ("Failed to create directory path.*/salt",
+            ('Failed to create directory path.*/salt',
                 perm_error_fmt.format(stderr)),
             ]
 
@@ -585,10 +762,26 @@ class Single(object):
                 return error[1]
         return None
 
-    def sls_seed(self, mods, env='base', test=None, exclude=None, **kwargs):
+    def sls_seed(self,
+                 mods,
+                 saltenv='base',
+                 test=None,
+                 exclude=None,
+                 env=None,
+                 **kwargs):
         '''
         Create the seed file for a state.sls run
         '''
+        if env is not None:
+            salt.utils.warn_until(
+                'Boron',
+                'Passing a salt environment should be done using \'saltenv\' '
+                'not \'env\'. This functionality will be removed in Salt '
+                'Boron.'
+            )
+        # Backwards compatibility
+        saltenv = env
+
         wrapper = salt.client.ssh.wrapper.FunctionWrapper(
                 self.opts,
                 self.id,
@@ -599,7 +792,7 @@ class Single(object):
         st_ = SSHHighState(minion_opts, pillar, wrapper)
         if isinstance(mods, str):
             mods = mods.split(',')
-        high, errors = st_.render_highstate({env: mods})
+        high, errors = st_.render_highstate({saltenv: mods})
         if exclude:
             if isinstance(exclude, str):
                 exclude = exclude.split(',')
@@ -624,8 +817,8 @@ class Single(object):
         trans_tar = prep_trans_tar(self.opts, chunks, file_refs)
         self.shell.send(
                 trans_tar,
-                '/tmp/salt_state.tgz')
-        self.arg_str = 'state.pkg /tmp/salt_state.tgz test={0}'.format(test)
+                '/tmp/.salt/salt_state.tgz')
+        self.arg_str = 'state.pkg /tmp/.salt/salt_state.tgz test={0}'.format(test)
 
 
 class SSHState(salt.state.State):
@@ -676,18 +869,20 @@ def lowstate_file_refs(chunks):
     '''
     refs = {}
     for chunk in chunks:
-        env = 'base'
+        saltenv = 'base'
         crefs = []
         for state in chunk:
             if state == '__env__':
-                env = chunk[state]
+                saltenv = chunk[state]
+            elif state == 'saltenv':
+                saltenv = chunk[state]
             elif state.startswith('__'):
                 continue
             crefs.extend(salt_refs(chunk[state]))
         if crefs:
-            if not env in refs:
-                refs[env] = []
-            refs[env].append(crefs)
+            if saltenv not in refs:
+                refs[saltenv] = []
+            refs[saltenv].append(crefs)
     return refs
 
 
@@ -719,14 +914,14 @@ def prep_trans_tar(opts, chunks, file_refs):
     lowfn = os.path.join(gendir, 'lowstate.json')
     with open(lowfn, 'w+') as fp_:
         fp_.write(json.dumps(chunks))
-    for env in file_refs:
-        env_root = os.path.join(gendir, env)
+    for saltenv in file_refs:
+        env_root = os.path.join(gendir, saltenv)
         if not os.path.isdir(env_root):
             os.makedirs(env_root)
-        for ref in file_refs[env]:
+        for ref in file_refs[saltenv]:
             for name in ref:
                 short = name[7:]
-                path = file_client.cache_file(name, env)
+                path = file_client.cache_file(name, saltenv)
                 if path:
                     tgt = os.path.join(env_root, short)
                     tgt_dir = os.path.dirname(tgt)
@@ -734,7 +929,7 @@ def prep_trans_tar(opts, chunks, file_refs):
                         os.makedirs(tgt_dir)
                     shutil.copy(path, tgt)
                     break
-                files = file_client.cache_dir(name, env, True)
+                files = file_client.cache_dir(name, saltenv, True)
                 if files:
                     for filename in files:
                         tgt = os.path.join(
