@@ -13,6 +13,7 @@ import hmac
 import shutil
 import hashlib
 import logging
+import traceback
 
 # Import third party libs
 try:
@@ -42,7 +43,7 @@ def dropfile(cachedir, user=None):
     dfn = os.path.join(cachedir, '.dfn')
 
     def ready():
-        """
+        '''
         Because MWorker._update_aes uses second-precision mtime
         to detect changes to the file, we must avoid writing two
         versions with the same mtime.
@@ -50,7 +51,7 @@ def dropfile(cachedir, user=None):
         Note that this only makes rapid updates in serial safe: concurrent
         updates could still both pass this check and then write two different
         keys with the same mtime.
-        """
+        '''
         try:
             stats = os.stat(dfn)
         except os.error:
@@ -65,7 +66,7 @@ def dropfile(cachedir, user=None):
                 return True
 
     while not ready():
-        log.warning("Waiting before writing {0}".format(dfn))
+        log.warning('Waiting before writing {0}'.format(dfn))
         time.sleep(1)
 
     aes = Crypticle.generate_key_string()
@@ -77,10 +78,10 @@ def dropfile(cachedir, user=None):
             import pwd
             uid = pwd.getpwnam(user).pw_uid
             os.chown(dfnt, uid, -1)
-            shutil.move(dfnt, dfn)
         except (KeyError, ImportError, OSError, IOError):
             pass
 
+    shutil.move(dfnt, dfn)
     os.umask(mask)
 
 
@@ -164,7 +165,7 @@ class MasterKeys(dict):
             log.info('Generating keys: {0}'.format(self.opts['pki_dir']))
             gen_keys(self.opts['pki_dir'],
                      'master',
-                     4096,
+                     self.opts['keysize'],
                      self.opts.get('user'))
             key = RSA.load_key(self.rsa_path)
         return key
@@ -218,7 +219,7 @@ class Auth(object):
             log.info('Generating keys: {0}'.format(self.opts['pki_dir']))
             gen_keys(self.opts['pki_dir'],
                      'minion',
-                     4096,
+                     self.opts['keysize'],
                      self.opts.get('user'))
             key = RSA.load_key(self.rsa_path)
         return key
@@ -265,7 +266,14 @@ class Auth(object):
         Pass in the encrypted aes key.
         Returns the decrypted aes seed key, a string
         '''
-        log.debug('Decrypting the current master AES key')
+        if self.opts.get('auth_trb', False):
+            log.warning(
+                    'Auth Called: {0}'.format(
+                        ''.join(traceback.format_stack())
+                        )
+                    )
+        else:
+            log.debug('Decrypting the current master AES key')
         key = self.get_keys()
         key_str = key.private_decrypt(payload['aes'], RSA.pkcs1_oaep_padding)
         if 'sig' in payload:
@@ -364,14 +372,23 @@ class Auth(object):
         if 'load' in payload:
             if 'ret' in payload['load']:
                 if not payload['load']['ret']:
-                    log.critical(
-                        'The Salt Master has rejected this minion\'s public '
-                        'key!\nTo repair this issue, delete the public key '
-                        'for this minion on the Salt Master and restart this '
-                        'minion.\nOr restart the Salt Master in open mode to '
-                        'clean out the keys. The Salt Minion will now exit.'
-                    )
-                    sys.exit(0)
+                    if self.opts['rejected_retry']:
+                        log.error(
+                            'The Salt Master has rejected this minion\'s public '
+                            'key.\nTo repair this issue, delete the public key '
+                            'for this minion on the Salt Master.\nThe Salt '
+                            'Minion will attempt to to re-authenicate.'
+                        )
+                        return 'retry'
+                    else:
+                        log.critical(
+                            'The Salt Master has rejected this minion\'s public '
+                            'key!\nTo repair this issue, delete the public key '
+                            'for this minion on the Salt Master and restart this '
+                            'minion.\nOr restart the Salt Master in open mode to '
+                            'clean out the keys. The Salt Minion will now exit.'
+                        )
+                        sys.exit(0)
                 else:
                     log.error(
                         'The Salt Master has cached the public key for this '
@@ -510,7 +527,7 @@ class SAuth(Auth):
         '''
         while True:
             creds = self.sign_in(
-                self.opts.get('_auth_timeout', 60),
+                self.opts['auth_timeout'],
                 self.opts.get('_safe_auth', True)
             )
             if creds == 'retry':
